/*
 * comdev.c - serial communication device
 * 
 * COM driver by Andrei Kovalev
 * Copyright (C) ZiLOG 2002
 * 
 * 12.08.2002 - com_init, com_open, com_close
 *  serial interrupt handler, combufferpool structure
 * 13.08.2002 - com_getc, com_putc, com_ctl(1), com_ctl(2)
 *  receiver test
 * 14.08.2002 12:15 - com_read, com_ctl(3)
 * 01.09.2002 21:48 - ordered two minor devices
 * ? - full function driver with COM1 and COM2
 */
#include <kernel.h>
#include <stdlib.h>
#include <shell.h>
#include <conf.h>
#include <sleep.h>

#include "com.h"

// ipworks serial module handling for versions
#define default_intr0 uziintr0
#define default_intr1 uziintr1

extern int comBfrSize;

#define COM_UNDEF ((void*)com_err)

void xprintf(char* x);
#define kprintf xprintf
#define DEBUG

int com_dev_init (struct devsw* pdev);
int com_open (struct devsw *pdev, char *nam, char *arg);
int com_close(struct devsw *pdev);
int com_ctl  (struct devsw *pdev, int func, char *addr, char *addr2);
int com_getc (struct devsw *pdev);
int com_putc (struct devsw *pdev, char ch);
int com_read (struct devsw *pdev, char *buff, int count);
int com_write(struct devsw *pdev, char *buff, int count);

void comhandler ( void );

/* entries for ipworks 1.4.7 */
//extern void uartintr0(void);
//extern void uartintr1(void);

/* empty comm function - for a feature that is not or cannot be implemented */
void com_err(void){}

const int null = 0;

tcom_device com_device[2];
               
/*--
 *   Format of the XINU device entries is:
 * device-number, device-name,
 * init, open, close,
 * read, write, seek,
 * getc, putc, cntl,
 * device-csr-address, input-vector, output-vector,
 * iint-handler, oint-handler, control-block, minor-device,
 */
struct devsw comdev = {   
        0, "COM", com_dev_init,
		com_open, com_close, com_read, com_write, COM_UNDEF,
        com_getc, com_putc, com_ctl,
		0000000, 0000, 0000, COM_UNDEF, COM_UNDEF, NULLPTR, 0};

/*--
 * The com_init function adds two COM devices, COM1 and COM2,
 * to the general XINU device table, initializes them and
 * thus lets the program to use the devices. While adding
 * a device it becomes an entry in the device table. Using
 * this entry number the device can be opened and used for
 * its predestinated purpose.
 * 
 * Normally in XINU the device is installed at compile time
 * when the device table is created. The device table index
 * is thus already known, such as UART0/UART1, or at least
 * for all master devices, for instance TCP or UDP.
 * 
 * This COM driver however would be installed at run time
 * and we need a method to give the program its handles.
 * The device table entry index will be returned by the
 * adddevice function after successful return. The device
 * table entry structure has a dvnum memeber which will be
 * properly initialized in the adddevice function:
 * 
 *  freedev->dvnum=nextfree;
 * 
 * This assigns the device the index of next available slot
 * in the device table. After that the device index is assigned
 * to the hCOM1 and hCOM2 handles correspondingly, so the
 * com_device array would contain the necessary data to use
 * those devices.
 */
int com_init( )
{
int comdevice;

    // Add the COM1 device to the XINU device table.
    if( (comdevice=adddevice(&comdev,0)) == SYSERR )
        kprintf("Unable to add COM1 device\n");
    else
    {
        // Init the first minor device.
        // Setup the device handle to be used by program.
        hCOM1 = comdevice;
        // The init function is called with devsw structure
        // as the parameter, which is the device table entry
        // setup by the adddevice function.
        devtab[comdevice]. dvinit(&devtab[comdevice]);
    }

    // Add the COM2 device to the XINU device table.
    if( (comdevice=adddevice(&comdev,1)) == SYSERR )
        kprintf("Unable to add COM2 device\n");
    else
    {
        // Init the second minor device
        hCOM2 = comdevice;
        devtab[comdevice]. dvinit(&devtab[comdevice]);
    }
}

/*--
 * The com_dev_init function receives the pointer to
 * the device table entry. There are two COM devices
 * available in eZ80. We do not do the separation between
 * a master device entry and device instances. In opposite
 * we create two entries for COM1 and COM2 and com_init
 * will be called for both of them.
 * 
 * The init function for the device instance resets the
 * communiction control block pointer.
 * 
 * @return OK
 */
int com_dev_init(struct devsw* pdev)
{
#ifdef DEBUG
    char msg[32];
    sprintf( msg, "com_init(%d)\n", pdev->dvnum );
    kprintf( msg );
#endif
    com_device[pdev->dvminor]. com_ctl_block = (void *)null;
}

/*--
 * The open function of the device receives the pointer to
 * the device table entry. The dvminor member differentiate
 * between COM1 and COM2. The name parameter is a logical
 * name of the device and is set by the program. The mode
 * parameter is not used in the current version of the driver.
 * We rely on IPWorks for any mode specific initialization.
 * 
 * So the main purpose of the open routine is to prepare
 * input and output buffers which are allocated dynamically.
 * 
 * @return index of the device in the device table
 */
int com_open(struct devsw* pdev, char *nam, char *mode)
{
unsigned char ctest, base;
struct combuffers *comio;
#ifdef DEBUG
    char msg[32];
    sprintf(msg,"com_open(%d,%d)\n", pdev->dvnum, pdev->dvminor );
    kprintf( msg );
    sprintf(msg,"opening %s\n", nam );
    kprintf( msg );
    sprintf(msg,"bfr size %d\n", comBfrSize );
    kprintf( msg );
#endif

    if( com_device[pdev->dvminor]. com_ctl_block != (combufferpool *)null )
        kprintf("akov-?-already open");
    else
    {
    // create the control block for the port
    combufferpool *com_ctl_block;
        com_ctl_block =  (combufferpool *)
            getmem(sizeof(combufferpool));
        // getmem returns a pointer to the mblock structure
        // create and initialize input and output buffers
        com_ctl_block->inbfr = (uc8*)getmem(comBfrSize);
        com_ctl_block->inrd  = com_ctl_block->inbfr;
        com_ctl_block->inwr  = com_ctl_block->inbfr;
        com_ctl_block->inov  = 0;
        com_ctl_block->oubfr = (uc8*)getmem(comBfrSize);
        com_ctl_block->ourd  = com_ctl_block->oubfr;
        com_ctl_block->ouwr  = com_ctl_block->oubfr;
        com_ctl_block->ouov  = 0;
        com_ctl_block->flow  = COM_FLOW_NONE;

        // assign the UART base
        com_ctl_block->base =
            (pdev->dvminor)
                ?0xD0       // for minor device 1 (com2)
                :0xC0;      // for minor device 0 (com1)

        // assign the control block to the device
        com_device[pdev->dvminor]. com_ctl_block = com_ctl_block;
#ifdef DEBUG
        memset((void*)com_ctl_block->inbfr, 0, comBfrSize);
        memset((void*)com_ctl_block->oubfr, 0, comBfrSize);
        sprintf(msg,"i/o %x|%x\n", com_ctl_block->inbfr, com_ctl_block->oubfr);
        kprintf( msg );
#endif
    }

    base = com_device[pdev->dvminor]. com_ctl_block -> base;
#ifdef DEBUG
    sprintf(msg,"base %x\n", base );
    kprintf( msg );
#endif

    /*
     * Initialize the UART. The baud rate and comm settings
     * are set by uzi module during the serial device
     * initialization. - UART_LCTL, DLRL, DLRH
     * 
     * Disable serial interrupt, set the FIFO control register
     * Read modem and line status
     * Enable the interrupt
     */

/*
 * NOTE:
 * compiler 2 cannot process internal i/o pointers to variables
 * the constants are needed and if expression to determine which
 * /
    if( base == UART1 )
    {
        UART_IER(UART1) = 0;
    // redirect the interrupt to our perfect handler
    if( pdev->dvminor )
    {
        set_evec( IV_UZI1, comhandler);
    }    
    else
    {
        set_evec( IV_UZI0, comhandler);
    }
    // enable FIFO, clear it and set the trigger level to 1 byte
    UART_FCTL(UART1) = 3;
    // read the status registers to reset any interrupt conditions
    ctest = UART_LSR(UART1);
    ctest = UART_MSR(UART1);
    // enable receiver interrupt    | IER_RIE = 0x01
    // enable transmitter interrupt | IER_TIE = 0x02
    UART_IER(UART1) = IER_RIE;
    }
    else
    {
        UART_IER(UART0) = 0;
    // enable FIFO, clear it and set the trigger level to 1 byte
    UART_FCTL(UART0) = 3;
    // read the status registers to reset any interrupt conditions
    ctest = UART_LSR(UART0);
    ctest = UART_MSR(UART0);
    // enable receiver interrupt    | IER_RIE = 0x01
    // enable transmitter interrupt | IER_TIE = 0x02
    UART_IER(UART0) = IER_RIE;
    }
	*/

    UART_IER(base) = 0;
    // redirect the interrupt to our perfect handler
    if( pdev->dvminor )
    {
#ifdef DEBUG
        sprintf( msg, "\ninstall i-vector %xH\n", IV_UZI1);
        kprintf( msg );
#endif
        set_evec( IV_UZI1, comhandler);
    }    
    else
    {
#ifdef DEBUG
        sprintf( msg, "\ninstall i-vector %xH", IV_UZI0);
        kprintf( msg );
#endif
        set_evec( IV_UZI0, comhandler);
    }
    UART_LCTL(base)   = 0x80;
    // init to default 57600
    #ifdef _EZ80190
    BRG_DLRL(base)    = 43;
    #else
    BRG_DLRL(base)    = 52;
    #endif
    BRG_DLRH(base)    = 0;
    UART_LCTL(base)   = 0x03;
    UART_FCTL(base)   = 0xC7;

// test the configuration:
//    UART_RBR(base)   = 0x0d;
//    UART_RBR(base)   = 0x0a;
//    UART_RBR(base)   = 't';
//    UART_RBR(base)   = 'e';
//    UART_RBR(base)   = 's';
//    UART_RBR(base)   = 't';

    // enable FIFO, clear it and set the trigger level to 1 byte
    //UART_FCTL(base) = 3;
    // read the status registers to reset any interrupt conditions
    ctest = UART_LSR(base);
    ctest = UART_MSR(base);
    // enable receiver interrupt    | IER_RIE = 0x01
    // enable transmitter interrupt | IER_TIE = 0x02
    UART_IER(base) = IER_RIE;


    return pdev->dvnum;
}

/*--
 * close com port
 */
int com_close(struct devsw *pdev)
{
    combufferpool *com_ctl_block;
#ifdef DEBUG
    char msg[32];
    sprintf(msg,"com_close(%d)\n", pdev->dvnum );
    kprintf( msg );
#endif

    com_ctl_block =
        com_device[pdev->dvminor]. com_ctl_block;

    // switch back to UZI interrupts
    UART_IER(com_ctl_block->base) = 0;

	#ifdef _EZ80190_ZDS1
    if( pdev->dvminor )
        set_evec( IV_UZI1, default_intr1);
    else
        set_evec( IV_UZI0, default_intr0);
	#endif

    UART_IER(com_ctl_block->base) = IER_RIE;

    freemem( (struct mblock *)(com_ctl_block->inbfr), comBfrSize);
    freemem( (struct mblock *)(com_ctl_block->oubfr), comBfrSize);
    freemem( (struct mblock *)(com_ctl_block), sizeof(combufferpool) );

    com_ctl_block = (combufferpool *)null;
}

extern unsigned long  masterclock;

/*--
 * control com port - set and reset parameters
 * 
 * pdev - device handle
 * func - control function ID
 * arg1 - argument one
 * arg2 - argument two
 */
int com_ctl(struct devsw *pdev, int func, char *arg1, char *arg2)
{
int return_value;
combufferpool *com_ctl_block;

    com_ctl_block =
        com_device[pdev->dvminor]. com_ctl_block;

    switch( func )
    {
        case GET_INQ :
            // return the number of chars available in input queue
            return_value = com_ctl_block->inwr - com_ctl_block->inrd;
            if( return_value < 0 )
                return_value = comBfrSize + return_value;
        break;
        case GET_OUQ :
            // return the number of chars ready for output
            return_value = com_ctl_block->ouwr - com_ctl_block->ourd;
            if( return_value < 0 )
                return_value = comBfrSize + return_value;
            // it is more important to reflect the available
            // space in the output buffer, invert this value
            // over BfrSize
            return_value = comBfrSize - return_value;
        break;
        case GET_OV :
            // return overflow count
            return_value = com_ctl_block->inov;
        break;
        case SET_RATE :
        {    
        unsigned char base, value;
        int baudrate;
            // serial baudrate configuration:
            // Divisor = SCLOCK/(16 * baudrate)
            base = com_ctl_block->base;
            baudrate = masterclock/(16*((int)arg1));
            UART_LCTL(base)   |= 0x80;
            // low byte
            value = (unsigned char)baudrate;
            BRG_DLRL (base)   = value;
            // high byte
            value = (unsigned char)(baudrate/256);
            BRG_DLRH (base)   = value;
            UART_LCTL(base)   &= ~0x80;
        }
		break;
        case SET_LINE :
        {    
        unsigned char base;
            // set line control register
            base = com_ctl_block->base;

            UART_LCTL(base)   = (unsigned char)arg1;
        }
		break;
        case SET_FLOW :
        {
            // set the flow parameter from the argument
            com_ctl_block->flow = (unsigned char)arg1;
            // if hardware flow control, set RTS - request to send!
            if( com_ctl_block->flow == COM_FLOW_HARD )
                UART_MCTL(0xC0) |= 0x3;
            else
            // software flow control, send the resume char
            if( com_ctl_block->flow == COM_FLOW_SOFT )
                com_putc(pdev,COM_XON);
        }
        break;
    }
    return return_value;
}

/*--
 * get a charachter from the com port
 */
int com_getc(struct devsw *pdev)
{
combufferpool *com_ctl_block;

    com_ctl_block =
        com_device[pdev->dvminor]. com_ctl_block;
    
    if( com_ctl_block->inrd - com_ctl_block->inbfr >= comBfrSize )
        com_ctl_block->inrd = com_ctl_block->inbfr;

    // no characters available
    if( com_ctl_block->inrd == com_ctl_block->inwr )
        return EOF;

    return *com_ctl_block->inrd++;
}

/*--
 * @func com_putc
 * v1.1 - still do not know how to control overflow here
 * control will be done in con_write as high level routine
 */
int com_putc(struct devsw *pdev, char ch)
{
combufferpool *com_ctl_block;

    com_ctl_block =
        com_device[pdev->dvminor]. com_ctl_block;
    
    *com_ctl_block->ouwr++ = ch;
    if( com_ctl_block->ouwr - com_ctl_block->oubfr >= comBfrSize )
        com_ctl_block->ouwr = com_ctl_block->oubfr;
    // enable the transmit interrupt for buffer handling
    // in the interrupt service routine
    UART_IER(com_ctl_block->base) |= IER_TIE;
}

/*--
 * @func com_read
 * get up to count chars from the input buffer
 * leaving when no more chars available
 * read pointer grows till it is equal to write pointer
 */
int com_read(struct devsw *pdev, char *buff, int count)
{
int len;
combufferpool *com_ctl_block;

    com_ctl_block =
        com_device[pdev->dvminor]. com_ctl_block;

    //com_ctl_block->inrd = com_ctl_block->inwr;
    len = com_ctl_block->inwr - com_ctl_block->inrd;

    // no chars available
    if( len == 0 )
        return EOF;

    len = 0;
    while ( len < count )
    {
        int ch;
        // com_getc processes buffer wrapping
        ch = com_getc( pdev );
        if( ch != EOF )
        {
            *buff ++ = ch;
            len ++;
        }
        else
            break;
    }
    return len;

    /*
    if( len < 0 )
    {
        // reading over the buffer bound
        // available 2048 + len chars
    }  
     * strcpy is for ascii transfers  
     * use memcpy
    strcpy( buff, com_ctl_block->inrd);
     */
}

/*--
 * @func com_write
 */
int com_write(struct devsw *pdev, char *buff, int count)
{
#ifdef DEBUG
char msg[64];
#endif
combufferpool *com_ctl_block;
int cnt = com_ctl( pdev, 2, 0, 0);
    if( com_ctl( pdev, 2, 0, 0) < count )
        return SYSERR;

    com_ctl_block =
        com_device[pdev->dvminor]. com_ctl_block;

#ifdef DEBUG0
    sprintf(msg, " COM%d: out %d\n", pdev->dvminor+1, cnt);
    kprintf(msg);
    {
        // test dump...
        int i;
        kprintf("\nwr: ");
        for ( i=0; i<count; i++)
        {
            msg[i] = buff[i];
        }
        kprintf(buff);
    }
#endif

    // write the buffer to UART using com_putc
    while( count -- )
        com_putc( pdev, *buff ++ );
}
