/*
 @doc HEADERFLASHLD
 @module FLASHLD.H |
 ********************************************************************
 *
 *	Header file for the FLASHLD.C module. This file defines structures
 *	and function prototypes used in the flash loader project.<nl>
 *	Furthermore, flash memory types will be defined here. All flash
 *	types defined in <t FLASH_TYPE> will be supported by this flash
 *	loader. Additional flash types can be included in order to support
 *	more or other flash memories from other manufacturer like Atmel,
 *	Intel... and so on.<nl>
 *	
 *
 *  
 *	HISTORY:													<nl>
 *	$Log: FLASHLD.H,v $
 *	05/21/01	57.6kB default, SW/default)/HW flow control, receive
 *				SPACE to force bootloader start	
 *
 ********************************************************************
 @index type struct enum | HEADERFLASHLD
 @normal Copyright (c) 2001 ZiLOG, Inc
 @end
 ********************************************************************
*/

#ifndef __FLASHLD_H__
#define __FLASHLD_H__
#include "stddefs.h"

/*******************************************************************
 * @type pIIO |
 * Defines a pointer to a internal 8Bit I/O register.
 *******************************************************************/
typedef volatile U8 __INTIO *__pIIO;

/*
 **************************************************************************************
 * eZ80 Chip Select register definition. For each Chip Select the dedicated register
 * set is available.
 **************************************************************************************
*/
#define CS0 0xA8
#define CS1 0xAB
#define CS2 0xAE
#define CS3 0xB1

#define	CSLBR(q)		(((__pIIO)(q + 0))[0])
#define	CSUBR(q)		(((__pIIO)(q + 1))[0])
#define	CSCTL(q)		(((__pIIO)(q + 2))[0])

/*
 *****************************************************************************
 * This defines the Base Address of the Flash memory. We need this address in 
 * order to access the flash physically. We will use the programmed value of the
 * corresponding CS LBR register. It depends on the CS what flash (in the case of 
 * more than 1) will be actually accessed.
 *****************************************************************************
 */
/*
 *******************************************************************************
 * This defines the CS configuration of the used HW platform. 
 *******************************************************************************
 */
#ifdef	EZ80EB_REV_C	//Crystal Board
#define	FLASH1_CS	CS0
#define FLASH2_CS	CS0
#define RAM1_CS		CS1
#define RAM2_CS		CS2
#define EMAC_CS		CS3
#else					//Realtek Board
#define	FLASH1_CS	CS0
#define FLASH2_CS	CS3
#define RAM1_CS		CS1
#define RAM2_CS		CS1
#define EMAC_CS		CS2
#endif	//EZ80EB_REV_C

//defines the flash base address. The current flash descriptor will be used
//#define FLASH_BASE_ADR	(((U32)(CSLBR(FLASH1_CS))) << 16 )
//#define DontCare		( (((U32)(CSLBR(FLASH1_CS))) << 16 ) + 0xAAAA)

#define FLASH_BASE_ADR	(pThisFlash->flBase)
#define DontCare        (pThisFlash->flBase + 0xAAAA)

/*
 *****************************************************************************
 * Defines the maximal number of blocks which can be cleared separately.
 * for flash types with maximal number of blocks of 11 when necessary this 
 * value can be adjusted
 ****************************************************************************
 */
#define MAX_NB_BLOCK 11	

/*
 *****************************************************************************
 * Defines the maximal number flash devices supported in that library. Thus,
 * this number defines the number of flash descriptors for a specific system 
 * that means that MAX_NB_FLSH_DEVICES can be incorporated in the hardware.
 * Increase the number when necessary.
 ****************************************************************************
 */
#define MAX_NB_FLASH_DEVICES 10

/*************************************************************************
 * @enum IHEX_FRAME_TYP |	
 * Intel HEX frame definition in order to distinguish between the different
 * frames.
 ************************************************************************/
typedef enum
{
	IHEX_TYP0 = 0,		//@emem Standard data frame type.
	IHEX_TYP1,			//@emem End of File frame.
	IHEX_TYP2,			//@emem Enhanced 64k Address control frame.
	IHEX_TYP3,			//@emem I don't no frame.
	IHEX_TYP4,			//@emem Enhanced linear Address control frame.
	IHEX_NO_TYP			//@emem This indicates a wrong frame type.
}IHEX_FRAME_TYP;

/*************************************************************************
 * @enum FLASH_TYPE |	
 * Definition of Flash memory types. When necessary, more types can be
 * defined here!<nl>
 * Note: The value 'NB_FLASH_SUPPORTED' must be the last entry to ensure
 *       proper operation!
 ************************************************************************/
typedef enum
{
	MT28F800B3_B,			//@emem Micron Technology MT28F800B3 Bottom Boot 8Mb flash.
	MT28F800B3_T,			//@emem Micron Technology MT28F800B3 Top Boot 8Mb flash.
	MT28F008B3_B,			//@emem Micron Technology MT28F008B3 Bottom Boot 8Mb flash.
	MT28F008B3_T,			//@emem Micron Technology MT28F008B3 Top Boot 8Mb flash.
	MT28F400B3_B,			//@emem Micron Technology MT28F400B3 Bottom Boot 4Mb flash.
	MT28F400B3_T,			//@emem Micron Technology MT28F400B3 Top Boot 4Mb flash.
	MT28F004B3_B,			//@emem Micron Technology MT28F004B3 Bottom Boot 4Mb flash.
	MT28F004B3_T,			//@emem Micron Technology MT28F004B3 Top Boot 4Mb flash.
	NB_FLASH_SUPPORTED,	//@emem Number of supported flash memory types.
	NO_FLASH					//@emem Value for NO FLASH as return value.
}FLASH_TYPE;

/*************************************************************************
 * @enum FLASH_SIZE	|
 * Definition of Flash memory size. This mask will be used when programming
 * the device. When the original base address within the Inel Hex file (Type 4)
 * indicates a address other than the base address used in the current 
 * Flash Loader's CS settings, this address must be adjusted to the current
 * CS settings in order to be able to access the flash memory physically.
 ************************************************************************/
typedef	enum
{
	SIZE_512KBYTE	= 0x0FFFFF,		//@emem 512kByte Flash Memory.
	SIZE_1MBYTE		= 0x0FFFFF,		//@emem 1MByte Flash Memory.
 	SIZE_2MBYTE		= 0x1FFFFF,		//@emem 2MByte Flash memory.
 	SIZE_4MBYTE		= 0x3FFFFF,		//@emem 4MByte Flash Memory.
	SIZE_8MBYTE		= 0x7FFFFF		//@emem 8MByte FLash Memory.
}FLASH_SIZE;

/*************************************************************************
 * @enum BLOCK_SIZE	|
 * Definition of the corresponding Block size. This will ber used when 
 * erasing a separate block or reading a whole block.
 ************************************************************************/
typedef	enum
{
	BSIZE_BOOT	= 0x4000,		//@emem 16kB protectable Boot Block.
	BSIZE_PARA1	= 0x2000,		//@emem 8kB first parameter block.
 	BSIZE_PARA2	= 0x2000,		//@emem 8kB second parameter block.
 	BSIZE_COMMON1	= 0x16000,	//@emem 96kB common data block.
	BSIZE_COMMON2	= 0x20000		//@emem 128kB common data block.
}BLOCK_SIZE;

/*************************************************************************
 * @enum FILE_TYPE	|
 * Defines file types which can be downloaded into the flash memory.
 ************************************************************************/
typedef	enum
{
	FILE_APPLICATION,		//@emem This is a Application file linked at 0x8000.
 	FILE_BOOTLOADER,		//@emem This is a Boot Loader file for the flash boot sector.
	FILE_WEBPAGE,			//@parm Web page file including web files and CGI's.
	FILE_BOOTINFO,			//@parm Boot Configuration file->Parameter Block 1.
	FILE_PERSONALITY,		//@parm Personality file->PArameter Block 1.
	FILE_WPLINKLIST		//@parm Webpage Link List file->Parameter Block 2
}FILE_TYPE;

/*************************************************************************
 * @enum MT_ERROR_CODE |	
 * Error code definition for the Micron Technology Flash memory.
 * This error code will be derived from the status register content.
 ************************************************************************/
typedef enum
{
	MT_NO_ERROR = 0,		//@emem No error occured. Command successful.
	MT_Vpp_ERROR,			//@emem Vpp voltage error.
	MT_WRITE_ERROR,		//@emem WRITE error.
	MT_WRITE_Vpp_ERROR,		//@emem WRITE error - Vpp not valid at time of write.
	MT_ERASE_ERROR,		//@emem ERASE error.
	MT_ERASE_Vpp_ERROR,		//@emem ERASE error - Vpp not valid at time of erase.
	MT_CMD_SEQ_ERROR,		//@emem Command sequencing error or WRITE/ERASE error.
	MT_CMD_SEQ_Vpp_ERROR,	//@emem Command sequencing error or WRITE/ERASE error and Vpp error.
	/*------------------*/	// further error codes fo internal use
	MT_WRONG_BLOCK_NUM,		//@emem Wrong block number.
	MT_FLASH_ID_ERROR,		//@emem Flash ID error.
	MT_NUM_ERROR			//@emem Maximal number of error codes.
}MT_ERROR_CODE;

/******************************************************************** 
 * @struct TypeFlashType | 
 * Structure with flash memory specific data!
 ********************************************************************/
typedef struct FlashType_tag
{
	FLASH_TYPE	FlashType;				//@field Flash memory of type <t FLASH_TYPE>.
	U16			FlashID;					//@field Flash ID containing Device ID (LSB) and Manufacture ID (MSB).
	U8			NoBlocks;					//@field Number of blocks.
	FLASH_SIZE	SizeMask;					//@field Flash size in order to adjust the address when writing data, see <t FLASH_SIZE>.
	U32			SectorSize;				//@field Maximal Sector size for programming the flash (not really necessary for Micron Technology).
	S8 			FlashName[17];				//@field Name of the Flash memory - max. 16 character.
	U32 			FlashBlock[MAX_NB_BLOCK+1];	//@field Flash block address table for this flash memory.
}TypeFlashType;

/*******************************************************************
 * @type pTypeFlashType |
 * Defines a pointer to the structure <t TypeFlashType>.
 *******************************************************************/
typedef TypeFlashType *pTypeFlashType;


/******************************************************************** 
 * @struct TypeFlashDescriptor | 
 * Structure for a flash descriptor which contains information about
 * where a physical flash device is located in a system. This structure
 * will be set by the application according to the corresponding platform.
 ********************************************************************/
typedef struct FlashDescriptor_tag
{
	U32			flBase;			//@field Base address of this flash device within the whole memory range.
	FLASH_SIZE	flSize;			//@field Flash size in order to adjust the address when writing data, see <t FLASH_SIZE>.
    /*
    U8              csBase;         //@field CS register base address where this flash is connected.
    U8              csLBR;          //@field CS LBR register address where this flash is connected.
    U8              csUBR;          //@field CS UBR register address where this flash is connected.
     */
}TypeFlashDescriptor;

/*******************************************************************
 * @type pTypeFlashDescriptor |
 * Defines a pointer to the structure <t TypeFlashDescriptor>.
 *******************************************************************/
typedef TypeFlashDescriptor *pTypeFlashDescriptor;

/******************************************************************** 
 * @struct TypeSysTable | 
 * Systam table containing the flash descriptors for each flash
 * device in the system. Maximal MAX_NB_FLASH_DEVICES can be set
 * in that system table.
 ********************************************************************/
typedef struct SysTable_tag
{
    U8                  nbEntry;                //@field Number of valid entries in that table.
	TypeFlashDescriptor	Descriptor[MAX_NB_FLASH_DEVICES];	//@field Array of flash descriptors.
}TypeSysTable;

/*******************************************************************
 * @type pTypeSysTable |
 * Defines a pointer to the structure <t TypeSysTable>.
 *******************************************************************/
typedef TypeSysTable *pTypeSysTable;


#endif //__FLASHLD_H__

/*
 **********************************************************************
 * Function prototypes and EXTERN declaration for other modules!	
 **********************************************************************
*/
__REFERENCE__ BOOL ReadFlashID(U32 FA);
__REFERENCE__ U8 GetFlashBlock(U32 FA);
__REFERENCE__ BOOL DownloadFile(FILE_TYPE File, U32 FA);
__REFERENCE__ void OutString(S8 *pstring);
__REFERENCE__ MT_ERROR_CODE EraseBlock(U8 BlockNumber);
__REFERENCE__ MT_ERROR_CODE EraseFlash(U8 FirstBlock, U8 LastBlock);
__REFERENCE__ U8 GetFlashType(void);
__REFERENCE__ BOOL CheckBlank(U8 FlashIndex, U8 BlkNum);
__REFERENCE__ BOOL GetHexNumber(U8 *pSourceAdr, U8 Count, U32 *pDestAdr);
__REFERENCE__ BOOL ErrorHandler(MT_ERROR_CODE ErrorCode);
__REFERENCE__ BOOL WriteBinFile(FILE_TYPE File, U32 FA, U32 NbByte, U8 *pSector);
__REFERENCE__ BOOL WriteBinData(U32 FA, U32 NbByte,	U8 *pSector);
__REFERENCE__ BOOL PrepareFlashUpdate(U32 FA, U32 FileSize);
__REFERENCE__ BOOL DownloadFileToFlash(void);
__REFERENCE__ BOOL GetNextChar(U8 *pDest);
__REFERENCE__ BOOL GetIHexFrame(U8 *pFrameBuffer, U16 Size);
__REFERENCE__ IHEX_FRAME_TYP ReadSector(U8 *pSector, U32 *pSectorSize, U32 *pSectorAddr);
__REFERENCE__ void ByteCopy(U24 dest, U24 src, U24 nByte);
__REFERENCE__ BOOL SetWpllInfo(U32	SA,	U32 	Size);
__REFERENCE__ BOOL GetDescriptor(U32	FA);
__REFERENCE__ BOOL FlashInit(U8 nb, TypeFlashDescriptor *pdesc);


