/*
 @doc HEADERFLASHSYS
 @module FLASHSYS.H |
 ********************************************************************
 *
 *	Header file for the FLASHSYS.C module. This module contains
 *	basic definitions and function prototypes.
 *	
 ********************************************************************
 @index type struct enum | HEADERFLASHSYS
 @normal Copyright (c) 2001 ZiLOG, Inc
 @end
 ********************************************************************
*/

#ifndef	__FLASHSYS_H__
#define	__FLASHSYS_H__
#include "stddefs.h"

#define ESC 0x1B
#define SPACE 0x20

//Define the command bytes in such a way that a write will work on each platform!
//For eZ80 eval board, the data bus on the flash is in reverse order. Thats why, before
//sending commands to the flash, the value must be reversed.
#ifdef EZ80EB_REV_C
#define ReadArray 			0xFF
#define IDDevice 			0x09
#define ReadStatusReg 		0x0E
#define ClearStatusReg 		0x0A
#define EraseSetup 			0x04
#define EraseConfirm 		0x0B
#define EraseSuspend 		0x0D
#define EraseResume 		0x0B
#define WriteSetup 			0x02
#define AltWordByteWrite 	0x08
#else
#define ReadArray 			0xFF
#define IDDevice 			0x90
#define ReadStatusReg 		0x70
#define ClearStatusReg 		0x50
#define EraseSetup 			0x20
#define EraseConfirm 		0xD0
#define EraseSuspend 		0xB0
#define EraseResume 		0xD0
#define WriteSetup 			0x40
#define AltWordByteWrite 	0x10
#endif

/*************************************************************************
 * Macro in order to create the reverse value of a byte.
 ************************************************************************/
#ifdef	EZ80EB_REV_C
#define	RevByte(BeforeRev)	(pRAMFunction->ReverseTable[ BeforeRev & 0x0F ] << 4 | pRAMFunction->ReverseTable[ BeforeRev >> 4 ])
#else
#define	RevByte(BeforeRev)	(BeforeRev)
#endif	//EZ80EB_REV_C

/*************************************************************************
 * @type TypeBlockEraseSeqFunction |	
 * BlockEraseSeq function used for erasing flash memory block.
 ************************************************************************/
typedef MT_ERROR_CODE (*TypeBlockEraseSeqFunction)(U32 BA);

/*************************************************************************
 * @type TypeWriteSequenceFunction |	
 * WriteSequence function used for programming flash memory blocks.
 ************************************************************************/
typedef MT_ERROR_CODE (*TypeWriteSequenceFunction)(U32 FA, U32 NoByte, U8 *pRam);

/*************************************************************************
 * @type TypeDeviceIDFunction |	
 * DeviceID function used to get the device ID.
 ************************************************************************/
typedef	U8 (*TypeDeviceIDFunction)(U32 FlashBase);

/*************************************************************************
 * @type TypeManufactureCompFunction |	
 * ManufactureComp function used to get the manufacture ID.
 ************************************************************************/
typedef	U8 (*TypeManufactureCompFunction)(U32 FlashBase);

/*************************************************************************
 * @type TypeReadBackVerifyFunction |	
 * ReadBackVerify function used for erasing flash memory block.
 ************************************************************************/
typedef	BOOL (*TypeReadBackVerifyFunction)(U32 FA, U32 NoByte, U8 *pRam);

/******************************************************************** 
 * @struct TypeRAMFunction | 
 * This structure contains function pointer for flash access which
 * are located in RAM. When using the Flash loader as resident 
 * flash loader, all flash routines must be copied into RAM and called
 * from that location.
 ********************************************************************/
typedef struct RAMFunction_tag
{
    TypeBlockEraseSeqFunction       stBlockEraseSeq;              //@field Block Erase Sequence.
    TypeWriteSequenceFunction       stWriteSequence;              //@field Write Sequence function.
    TypeDeviceIDFunction            stDeviceID;                   //@field Get Device ID.
    TypeManufactureCompFunction     stManufactureComp;            //@field Get Manufacture ID.
    TypeReadBackVerifyFunction      stReadBackVerify;             //@field Read back and Verify function.
	#ifdef	EZ80EB_REV_C
	U8											ReverseTable[16];
	#endif	//EZ80EB_REV_C
}TypeRAMFunction;

/*******************************************************************
 * @type pTypeRAMFunction |
 * Defines a pointer to the structure <t TypeRAMFunction>.
 *******************************************************************/
typedef TypeRAMFunction *pTypeRAMFunction;

/*
 *****************************************************************************************
 * Macros defining flash system functions. When the flash loader will be downloaded into
 * the RAM, the standard function will be called. When the flash loader has been residently
 * stored in a flash boot sector, all flash sys functions must be copied to a RAM location
 * and must be called from RAM.
 *****************************************************************************************
 */
#define BlockEraseSeq(BA)               pRAMFunction->stBlockEraseSeq(BA)
#define WriteSequence(FA, Nbyte, pRam)  pRAMFunction->stWriteSequence(FA, Nbyte, pRam)
#define DeviceID(BaseAdr)               pRAMFunction->stDeviceID(BaseAdr)
#define ManufactureComp(BaseAdr)        pRAMFunction->stManufactureComp(BaseAdr)
#define ReadBackVerify(FA, Nbyte, pRam) pRAMFunction->stReadBackVerify(FA, Nbyte, pRam)

#endif	//_FLASHSYS_H_

/*
 ******************************************************************************************
 *	Function definition for intern and extern! The eZ80 C-Compiler will use this sequence
 *	to assign the address! Thus, the sequence of this prototypes must be the same like in
 *	the C-module. This is needed for copying all functions into RAM when using the flash
 *	loader as resifent boot loader!
 ******************************************************************************************
 */
__REFERENCE__ void RAMFunctionStart(void);
__REFERENCE__ MT_ERROR_CODE mtBlockEraseSeq(U32 BA);
__REFERENCE__ MT_ERROR_CODE mtWriteSequence(U32 FlashAddr, U32 NoBytes, U8 *pRamAddr);
__REFERENCE__ U8 mtDeviceID(U32 FlashBase);
__REFERENCE__ U8 mtManufactureComp(U32 FlashBase);
__REFERENCE__ BOOL mtReadBackVerify(U32 FlashAddr, U32 NoBytes, U8 *pRamAddr);
__REFERENCE__ void RAMFunctionEnd(void);
__REFERENCE__ void ConfigureFlashRAMFunction(void);


