/*
 @doc HEADERSERVCONFIG
 @module SERVCONFIG.H |
 *******************************************************************************
 *
 *	Header file for the 'ServConfig.c' source file. This file contains all 
 *	structures, types and prototypes for the configuration module. 			<nl>
 *	
 *******************************************************************************
 @index type struct enum | HEADERSERVCONFIG
 @normal Copyright (c) 2001 ZiLOG, Inc
 @end
 *******************************************************************************
*/

#ifndef __SERVCONFIG_H__
#define __SERVCONFIG_H__

#include <bootinfo.h>

#include "stddefs.h"
#include "Target.h"

/*
 *****************************************************************************
 * Defines a valid boot information block.
 ****************************************************************************
 */
#define		BOOT_INFO_VALID		0x1234		//indicates that the flash file is valid
#define 	BOOT_INFO_VALID_DEF	0x4321		//indicates that this is a valid default configuration
#define		FILE_VALID			0x1234		//indicates that the flash file is valid
#define 	FILE_VALID_DEF		0x4321		//indicates that this is a valid default configuration

// _DEFAULT_HOME flag: TRUE: always init http with _default_website structure

/*
 *****************************************************************************
 * Defines the start value for the 32bit Check Sum.
 ****************************************************************************
 */
#define CRC_MAGIC   0x00AFFE00

/*************************************************************************
 * @enum BOOT_INFO_TYP |	
 * Type definition in order to distinguish between a certain part within
 * the boot information block.
 ************************************************************************/
typedef enum
{
	BIT_BOOTRECORD = 0,	//@emem Boot Record field.
	BIT_SYSINFO,			//@emem System Information field.
	BIT_NETCONFIG,			//@emem Network configuration.
	BIT_ALL					//@parm All parts of the boot information block.
}BOOT_INFO_TYP;

/*************************************************************************
 * @enum BOOT_RECORD_TYP |	
 * Type definition for Boot Record IDs in order to change separate
 * elements by calling <f ChangeBootRecord>.
 ************************************************************************/
typedef enum
{
	BRT_IPADR = 0,			//@emem IP address.
	BRT_DEFGATEWAY,		//@emem Default Gateway.
	BRT_RFSERVER,			//@emem Default File Server.
	BRT_TFTPSERVER,		//@parm Default TFTP Server.
	BRT_DNSSERVER,			//@emem Default DNS Server.
	BRT_TIMESERVER,		//@emem Default Time Server.
	BRT_BOOTFILE,			//@emem Boot File name.
	BRT_SUBNETMASK		//@parm Subnet mask value.
}BOOT_RECORD_TYP;

/*************************************************************************
 * @enum SYS_CONFIG_TYP |	
 * Type definition for system configuration IDs in order to change 
 * separate elements by calling <f ChangeSystemConfiguration>.
 ************************************************************************/
typedef enum
{
	SCT_WPLINKLIST = 0		//@emem Indicates the Webpage Link List.
}SYS_CONFIG_TYP;

/*************************************************************************
 * @enum NET_CONFIG_TYP |	
 * Type definition for network configuration IDs in order to change 
 * separate elements by calling <f ChangeNetworkConfiguration>.
 ************************************************************************/
typedef enum
{
	NCT_DHCP = 0,			//@emem Indicates DCHP.
	NCT_HTTP,				//@emem Indicates HTTP.
	NCT_IGMP,				//@emem Indicates IGMP.
	NCT_SNMP,				//@emem Indicates SNMP.
	NCT_SMTP,				//@parm Indicates SMTP.
	NCT_TFTP,				//@emem Indicates TFTP.
	NCT_FTP,				//@emem Indicates FTP.
	NCT_PPPSERVER,			//@emem Indicates PPP Server.
	NCT_PPPCLIENT,			//@parm Indicates PPP Client.
	NCT_DCCSERVER,			//@emem Indicates PPP DCC Server.
	NCT_DCCCLIENT,			//@parm Indicates PPP DCC Client.
	NCT_TIMESERV,			//@emem Indicates Time Service.
	NCT_TELNET,				//@emem Indicates Telnet.
	NCT_SHELL				//@parm Indicates system Shell.
}NET_CONFIG_TYP;

/*************************************************************************
 * @enum PROTOCOL_STATUS_TYP |	
 * Type definition for protocol status. A variable from that type
 * indicates if the corresponding protocol has been enabled, disabled or
 * is not supported.
 ************************************************************************/
typedef enum
{
	PST_DISABLED = 0,		//@emem Indicates that the corresponding protocol has been disabled.
	PST_ENABLED,			//@emem Indicates that the corresponding protocol has been enabled.
	PST_NOT_SUPPORTED		//@emem Indicates that the corresponding protocol is not supported.
}PROTOCOL_STATUS_TYP;

/******************************************************************************
 * @struct TypeBootRecord | 
 * Structure for the Server's Boot Record. This field contains all information
 * needed for network access. This record will be overwritten by DHCP. In the
 * case DHCP is disabled, this record must contain a valid subnet IP address
 * and mask value. All other elements can be set.
 ******************************************************************************/
typedef struct BootRecord_tag
{
	S8		IPadr[32];				//@field IP address of the embedded system.
	S8		DefGateway[32];		//@field IP address of the Default Gateway.
	S8		TimeServer[32];		//@field IP address of the Time Server.
	S8		RFserver[32];			//@field IP address of the RF Server.
	S8		TFTPserver[32];		//@field IP address of the TFP Server.
	S8		DNSserver[32];			//@field IP address of the DNS Server.
	S8		BootFile[64];			//@field Name of the Boot File.
	S8		SubnetMask[32];		//@field Corresponding Subnet mask.
}TypeBootRecord;

/*******************************************************************
 * @type pTypeBootRecord |
 * Defines a pointer to the structure <t TypeBootRecord>.
 *******************************************************************/
typedef TypeBootRecord *pTypeBootRecord;

/******************************************************************************
 * @struct TypeSysInfo | 
 * Structure for System information. This field contains all needed information
 * in order to separate the Server base firmware and the web application (web
 * files, CGI functions).
 ******************************************************************************/
typedef struct SysInfo_tag
{
	U32		WpLinkList;				//@field Web page Link List address.
}TypeSysInfo;

/*******************************************************************
 * @type pTypeSysInfo |
 * Defines a pointer to the structure <t TypeSysInfo>.
 *******************************************************************/
typedef TypeSysInfo *pTypeSysInfo;

/******************************************************************** 
 * @struct TypeNetConfig | 
 * Structure for the Network Configuration. This field contains
 * the protocols with which the IPW Server should start. Every
 * time the server boots up, he load this configuration and 
 * initializes all the protocols which have been selected.
 ********************************************************************/
typedef struct NetConfig_tag
{
	PROTOCOL_STATUS_TYP	NC_DHCP;			//@field Using DHCP/BootTP module.
	PROTOCOL_STATUS_TYP NC_HTTP;			//@field Using HTTP module.
	PROTOCOL_STATUS_TYP	NC_IGMP;			//@field Using IGMP module.
	PROTOCOL_STATUS_TYP NC_SNMP;			//@field Using SNMP module.
	PROTOCOL_STATUS_TYP NC_SMTP;			//@field Using SMTP module.
	PROTOCOL_STATUS_TYP	NC_TFTP;			//@fiels Using TFTP module.
	PROTOCOL_STATUS_TYP	NC_FTP;				//@field Using FTP module.
	PROTOCOL_STATUS_TYP	NC_PPPSERVER;		//@field Using PPP Server module.
	PROTOCOL_STATUS_TYP	NC_PPPCLIENT;		//@field Using PPP Client module.
	PROTOCOL_STATUS_TYP	NC_DCCSERVER;		//@field Using PPP DCC Server module.
	PROTOCOL_STATUS_TYP	NC_DCCCLIENT;		//@field Using PPP DCC Client module.
	PROTOCOL_STATUS_TYP NC_TimeService;		//@field Using Time Service.
	PROTOCOL_STATUS_TYP NC_Telnet;			//@field Using Telnet module.
	PROTOCOL_STATUS_TYP NC_Shell;			//@field Using Shell module.
}TypeNetConfig;

/*******************************************************************
 * @type pTypeNetConfig |
 * Defines a pointer to the structure <t TypeNetConfig>.
 *******************************************************************/
typedef TypeNetConfig *pTypeNetConfig;

/******************************************************************** 
 * @struct TypeMulticastAddress | 
 * Structure for the Multicast Group. Each Server can belong to one
 * or more multicast groups. Once IGMP has been enabled, the server
 * will join all groups from that structure.
 ********************************************************************/
typedef struct MulticastAddress_tag
{
	S8		Address[32];	//@field Multicast Group address.
}TypeMulticastAddress;

/*******************************************************************
 * @type pTypeMulticastAddress |
 * Defines a pointer to the structure <t TypeMulticastAddress>.
 *******************************************************************/
typedef TypeMulticastAddress *pTypeMulticastAddress;

/*******************************************************************
 * Maximal number of Multicast Addresses in one goup.
 *******************************************************************/
#define MAX_NB_MULTICAST	4

/******************************************************************** 
 * @struct TypeMulticastGroup | 
 * Structure for the Multicast Group. Each Server can belong to one
 * or more multicast groups. Once IGMP has been enabled, the server
 * will join all groups from that structure.
 ********************************************************************/
typedef struct MulticastGroup_tag
{
	U8						Number;			//field Number of Multicast Group Addresses.
	TypeMulticastAddress	MCA[MAX_NB_MULTICAST];	//@field Multicast Address area.
}TypeMulticastGroup;

/*******************************************************************
 * @type pTypeMulticastGroup |
 * Defines a pointer to the structure <t TypeMulticastGroup>.
 *******************************************************************/
typedef TypeMulticastGroup *pTypeMulticastGroup;


/******************************************************************** 
 * @struct TypeBootInfo | 
 * Structure for the Boot Information. This information will be loaded
 * after power up from Flash or EEPROM and used to configure the 
 * system. Also, this information can bw written back to re-configure
 * this system.
 ********************************************************************/
typedef struct BootInfo_tag
{
	U16					valid;			//field Flag indicates that the info is valid.
	struct BootInfo	BootRecord;			//@field Boot Record entry.
	TypeSysInfo			SysInfo;		//@field System information.
	TypeNetConfig   	NetConfig;		//@field Network configuration.
	TypeMulticastGroup	MulticastGroup;	//field Multicast Group for one server.
	U32					CheckSum;		//@field 32 bit Check Sum.
}TypeBootInfo;

/*******************************************************************
 * @type pTypeBootInfo |
 * Defines a pointer to the structure <t TypeBootInfo>.
 *******************************************************************/
typedef TypeBootInfo *pTypeBootInfo;

/******************************************************************** 
 * @struct TypePersonality | 
 * Structure for the Personality Information. This information usually will be
 * located in the flash memory and can be loaded on request. It contains
 * basic information about the system and device which can be used for
 * different functions and to distinguish this device in a real network.
 ********************************************************************/
typedef struct Personality_tag
{
	U16					valid;				//field Indicates whether or not the file is valid.
	S8					DeviceName[32];		//field Device Identification as string.
	U32					CheckSum;			//field 23 bit check sum
}TypePersonality;

/*******************************************************************
 * @type pTypePersonality |
 * Defines a pointer to the structure <t TypePersonality>.
 *******************************************************************/
typedef TypePersonality *pTypePersonality;

#endif //__SERVCONFIG_H__

/*
 ******************************************************************************************
 *	Function definition for intern and extern! The eZ80 C-Compiler will use this sequence
 *	to assign the address! Thus, the sequence of this prototypes must be the same like in
 *	the C-module. This is needed for copying all functions into RAM when using the flash
 *	loader as resifent boot loader!
 ******************************************************************************************
 */
__REFERENCE__ BOOL LoadConfigFile(U32 FA, pTypeBootInfo	pBI);
__REFERENCE__ BOOL LoadPersonality(U32 FA, pTypePersonality	pPerson);
__REFERENCE__ BOOL SaveConfigFile(U32 FA, pTypeBootInfo pBI);
__REFERENCE__ BOOL SavePersonality(U32 FA, pTypePersonality pPerson);
__REFERENCE__ BOOL VerifyCRC(U32 SA, U32 EA);
__REFERENCE__ U32 CalcCRC(U32 SA, U32 EA);
__REFERENCE__ void SetpBootInfo(void);
__REFERENCE__ void SetpPersonality(void);
__REFERENCE__ pTypeBootInfo GetpBootInfo(void);
__REFERENCE__ pTypePersonality GetpPersonality(void);
__REFERENCE__ BOOL ReadConfigFile(pTypeBootInfo	pDest);	
__REFERENCE__ BOOL ReadPersonality(pTypePersonality pDest);	
__REFERENCE__ BOOL UpdateConfigFile(BOOL Save, pTypeBootInfo pSource);
__REFERENCE__ BOOL UpdatePersonality(BOOL Save, pTypePersonality pSource);
__REFERENCE__ BOOL ChangeBootRecord(BOOT_RECORD_TYP	brid, S8 *Strg);
__REFERENCE__ BOOL ChangeNetworkConfiguration(NET_CONFIG_TYP ncid, PROTOCOL_STATUS_TYP	bs);
__REFERENCE__ BOOL ChangeSystemConfiguration(SYS_CONFIG_TYP scid, U32 lVal);
__REFERENCE__ U32  DotNotationToU32(S8 *Strg);

