/*
 * Copyright 2001, Metro Link, Inc.
 * All Rights Reserved
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Metro Link, Inc. and may
 * contain proprietary, confidential and trade secret information of
 * Metro Link, Inc. and/or its partners.
 *  
 * The contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Metro Link, Inc.
 *
 */
#ifndef	_UPNP_H
#define	_UPNP_H	1

#ifdef unix
#include <assert.h>
#include <unistd.h>
#include <limits.h>
#include <time.h>

typedef enum { FALSE = 0, TRUE = 1 } Boolean;
#endif
#ifdef xinu
#define assert(foo) /**/
typedef int Boolean;
#endif

#ifdef VXWORKS
#include <vxWorks.h>
typedef int Boolean;
#endif

#include "httpd.h"

typedef char *UUID;

/*  Putting the typedefs up front allows us to arrange the structures
 *  in any desired order, which is useful for documentation purposes.
 */
typedef struct soap_request Soap_Request;
typedef struct gena_states Gena_States;
typedef const struct upnp_icon Upnp_Icon;
typedef const struct upnp_value Upnp_Value;
typedef const struct upnp_state Upnp_State;
typedef const struct upnp_action_arg Upnp_Action_Arg;
typedef const struct upnp_action Upnp_Action;
typedef const struct upnp_service Upnp_Service;
typedef const struct upnp_device Upnp_Device;
typedef struct upnp_device_root Upnp_Device_Root;


/** The gena_states structure.
 *  This structure is used to pass events to the send_event routine.  It is
 *  typedefed to be Gena_States.
 *  @name Gena_States
 *  @type typedef struct gena_states
 *
 *  @see gena_send_event
 */

struct gena_states {
    /** Pointer to the list of states and their associated values.  This is a
     *  list of 'key', 'value' pairs and is terminated by an entry where
     *  the key field is a null pointer. This list should never be empty, nor
     *  should the pointer ever be null. */
    Http_Params *state_list;

    /** Routine to be called whenever the event has been sent to free this
     *  memory.  The routine returns 'void' and accepts a pointer to a
     *  state_list above.  This field may be null if there is nothing to do to
     *  free this space. */
    void (*destroy) (Http_Params * states);
};

/** The soap request structure is passed to the action callback routines.
 *  However, the internals of this structure are subject to change.  It should
 *  only be accessed through the convenience routines provided.
 *
 *  @name Soap_Request
 *  @type typedef struct soap_request
 *  @see soap_get_actionid
 *  @see soap_get_queryStateVariable
 *  @see soap_get_actionArgument
 *  @see soap_get_user_data
 */
struct soap_request {
    Http_Request request;
    Upnp_Device **devices;
    Http_Params *arguments;
    char flags;
    Upnp_Service *service;
    Upnp_Action *action;
    int parse_state;
};


/**
 *  @memo This structure contains the information that describes an Icon
 *  graphic.
 *  @doc An example of a static upnp_icon would resemble the following:
 *  @name Upnp_Icon
 *  @type typedef const struct upnp_icon
*/
struct upnp_icon {
    /** The mime type of the icon image.  Typically, this will be "image/gif",
     * "image/jpg", or "image/png". */
    char *mimetype;

    /** The width of the image.  */
    int width;

    /** The height of the image.  */
    int height;

    /** The depth of the image. Typical values are 8 & 16.  */
    int depth;

    /** The web site that a control point can request to obtain the image.
     *  Typically, this is found on the device itself and is relative to the
     *  base URL. */
    Webpage *icon_URL;
};

/**
 *  A upnp_value struct defines the possible values of a UPnP state variable.
 *  The value is either a null terminated list of strings OR a numeric (in
 *  string form) maximum, minimum and (optional)step.
 *  @name Upnp_Value
 *  @type typedef const struct upnp_value
 */
struct upnp_value {
    /** A list of possible values for this variable. This list is null
     *  terminated.  If this value is described by a minimum/maximum pair, this
     *  pointer should be null. */
    char **list;

    /** The minimum(inclusive) value allowed for this variable. This is a string
     *  representation of the numeric value. */
    char *minimum;

    /** The maximum(inclusive) value allowed for this variable. This is a string
     *  representation of the numeric value. */
    char *maximum;

    /** The step used to increment or decrement this variable.  This is a
     *  string representation of the numeric value. */
    char *step;
};

/**
 *  A upnp_state struct describes a UPnP state variable.  The information here
 *  is used to in the service description sent to a control point.
 *  @name Upnp_State
 *  @type typedef const struct upnp_state
 */
struct upnp_state {
    /** The name of the state variable.  Non-standard variables must begin with
     * 'X_', according to the UPnP architectural spec. */
    char *name;

    /** Whether changes in this variable will generate an event. */
    Boolean evented;

    /** The data type of this state variable.  All variables are sent as
     * strings in the generated XML.  This field specifies one of the 23(24)
     * possible data types listed in the UPnP architectural spec. */
    char *dataType;

    /** The expected initial value for this variable.  It must match the data
     *  type and it must satisfy the allowedValueList or allowedValueRange
     *  constraints. Recommended.  If no default value is presented, this field
     *  should be null. */
    char *defaultValue;

    /** A pointer to the allowed value structure.  Recommended.  If there are
     * no specified allowed values, this field should be null. */
    Upnp_Value *allowed;
};

typedef enum { IN = 0, OUT = 1 } ArgDirection;

/** 
 *  A upnp_action_arg struct describes the arguments to a upnp action.  This
 *  includes the name, the related state variable and the 'direction' of the
 *  argument.  Each argument is related to some state variable in the service.
 *  This is mainly a 'typing' mechanism, since some arguments are not really
 *  related to any particular variable, while others may be related to several.
 *  @name Upnp_Action_Arg
 *  @type typedef const struct upnp_action_arg
 */
struct upnp_action_arg {
    /** The name of the formal parameter. It must not contain a hyphen
     * character.  It should be < 32 characters. */
    char *name;
    /** A pointer to the related state variable for this action.  Used for
     * typing. */
    Upnp_State *relatedVar;
    /** Whether this argument is input or a return value for the action.  The
     *  possible values are 'IN' and 'OUT', which are defined as an enum.
     */
    ArgDirection direction;
};

#define NO_ARGS (Upnp_Action_Arg **)0

/** The upnp_action structure defines an action.  This consists of a user
 *  defined id, a name, and an argument list.  In addition, there is a flag for
 *  whether this action returns a value.
 *  @name Upnp_Action
 *  @type typedef const struct upnp_action
 */
struct upnp_action {
    /** An integer defined by the user to identify this particular action.
     *  This id is only for the device implementor's convenience.  It allows
     *  the implementor to construct a switch statement based on this id in the
     *  action callback routine.  It must be a positive integer.
     *  @see soap_get_actionid
     */
    int actionid;
    /** The name of the action.  Non-standard actions must begin with 'X_',
     *  according to the UPnP architectural spec. They must also contain no
     *  hyphen character and should be < 32 characters long. */
    char *name;
    /** A pointer to a list of arguments for this particular action.  This must
     *  be a null terminated list.  If their are no arguments for this action,
     *  this field should be set to NO_ARGS.
     *  @see upnp_action_arg
     */
    Upnp_Action_Arg **arguments;
    /** Whether this particular action has a return value.  There can be at
     *  most one return value per action.  It must be the first OUT argument.
     *  If there are no out arguments, this field is ignored.
     */
    Boolean hasRetval;
};

/**
 *  The upnp_service structure defines a service provided by the device.  The
 *  implemetor fills in this structure.  The information contained here will be
 *  presented in the service description XML page.
 *  @name Upnp_Service
 *  @type typedef const struct upnp_service
 */
struct upnp_service {
    /** A non-standard schema.  If this is a non-standard device, the device
     *  implementor must enter their own schema consisting of a domain owned by
     *  the vendor.  (For example, metrolink.com).  This should be null if
     *  standard and will default to the string found in the architectural
     *  spec. It is used for the service type and the service id. */
    char *schema;

    /** The string representing the service type of this service.  It is of the
     *  format 'service suffix:version', e.g. tvcontrol:1.  Note that this
     *  excludes the schema. */
    char *serviceType;

    /** The string representing the service id of this service.  It is only the
     *  suffix portion if the id, that is it excludes the 'schema'.  It is
     *  limited to 64 characters. */
    char *serviceId;

    /** Pointer to a webpage structure to get the description of this
     *  service.  An example of the webpage entry for this URL is
     \begin{verbatim} {HTTP_PAGE_DYNAMIC, "/tvcontrolSCPD.xml", "text/xml", {upnpdev_get_service}}\end{verbatim}
     *  This entry should never be empty.
     */
    Webpage *service_Desc_URL;

    /** Pointer to a webpage structure to which SOAP requests should be
     *  sent for this service.  An example of the webpage entry for a soap URL
     *  is
     \begin{verbatim} {HTTP_PAGE_STATIC, "/control/tvcontrol", "text/xml", {NULL}}\end{verbatim}
     *  This entry should never be empty.
     */
    Webpage *control_URL;

    /** Pointer to a webpage structure to which GENA subscription requests
     *  should be sent for this service.  An example of the webpage entry for a
     *  soap URL is
     \begin{verbatim} {HTTP_PAGE_DYNAMIC, "/event/tvcontrol", "text/xml", {gena_subscription_service}}\end{verbatim}
     *  If there are no evented variables, this pointer should be null.
     */
    Webpage *event_Sub_URL;

    /** The list of actions which this service can perform. */
    Upnp_Action **actions;

    /** The list of state variables used by this service. */
    Upnp_State **states;

    /** The user provided routine called when a new subscription arrives to
     *  get the current state of all of the state variables. */
    void (*stateCallback) (void *serviceId, Gena_States *states);

    /** The user provided routine called whenever a SOAP action is received
     *  for this service.  The only parameter is a Soap_Request structure.
     *  This struture may be queried to determine the action called, as
     *  well as any parameters to that action. */
    void (*actionCallback) (Soap_Request * request);

    /** Any data the implemetor wishes to associate with this service.
     *  This can be accessed through the Soap_Request passed to the action
     *  callback. */
    void *user_data;
};

/**
 *  The upnp_device structure defines a device.  The implemetor fills in this
 *  structure.  The information contained here will be presented in the device
 *  description XML page.
 *  @name Upnp_Device
 *  @type typedef const struct upnp_device
 */
struct upnp_device {
    /** The non-standard schema (e.g. metrolink.com).  Empty if if this is a
     *  standard device (i.e. schemas-upnp-org). */
    char *schema;

    /** The string representing the device type of this device.  This field
     *  consists of the device type suffix followed by a colon followed by an
     *  integer version.  The schema is excluded.  The device type suffix
     *  should be <= 64 characters, not including the version and separating
     *  colon. */
    char *device_type;

    /** The short description of this device such that it can be recognized
     *  by an end user (e.g. UPnP Television Emulator).  Should be < 64
     *  characters.  Required. */
    char *friendly_name;

    /** The manufacturer of the device.  Should be < 64 characters.
     *  Required. */
    char *manufacturer;

    /** The web site of the manufacturer of the device.  Optional. */
    char *manufacturer_URL;

    /** The longer desciption for the end user.  Should be < 128 characters.
     *  Recommended. */
    char *model_description;

    /** The model name. Required. */
    char *model_name;

    /** The model number.  Should be < 32 characters. Recommended. */
    char *model_number;

    /** The web site for this specific model.  Optional. */
    char *model_URL;

    /** The serial number.  Should be < 64 characters. Recommended. */
    char *serial_number;

    /** The unique device name, in string format.  This Universally-unique
     * identifier for the device is generated by the software.  However,
     * the user may enter a 'prefix' identifier of some sort.  This may be
     * up to 20 hex digits. The last 12 characters should be left for the
     * device itself. This pointer should point to a, possibly initialized as
     * described above, space of 33 characters.  */
    char *UDN;

    /** The manufacturer's UPC number. */
    char *UPC;

    /** The web site from which this description can be obtained--e.g.
     *  '/tvdevicedesc.html'.   This description is generated by the
     *  enableworks libraries when this entry looks similar to the following:
     \begin{verbatim} {HTTP_PAGE_DYNAMIC, "/tvdevicedesc.xml", "text/xml", {upnpdev_get_device}}\end{verbatim}
     *  This indicates that the page is dynamically generated, has a URL of
     *  <baseAddress>/tvdevicedesc.xml, a mime type of 'text/xml' and is
     *  generated by the upnpdev_get_device() routine.  This routine is
     *  provided by the libraries and will produce an XML based on this
     *  device stucture.  Using some other routine is possible, but is
     *  discouraged unless there are some non-standard elements which must be
     *  presented.
     */
    Webpage *description_URL;

    /** The web site from which this description presentation page can be
     *  obtained--e.g. '/tvdevicepres.html'.  This webpage entry will look
     *  something like the following for a standard device:
     \begin{verbatim} {HTTP_PAGE_STATIC, "/tvdevicepres.html", "text/html", {&tvdevicepres_html}}\end{verbatim}
     *  This indicates that the page is static, has a URL of
     *  <baseAddress>/tvdevicedesc.xml, a mime type of 'text/xml' and is
     *  found in the character array at tvdevicepres_html.  This array can be
     *  produced by the mkwebpage script shipped with enable works.
     *  
     */
    Webpage *presentation_URL;

    /** Flag for determining whether this is a root device.  Should not be
     *  specified by the user. */
    Boolean is_root_device;

    /** Pointer to a the list of Upnp_Icons belonging to this device.
     *  This list a null terminated list.  If there are no icons provided by
     *  this device, this pointer should be set to null. */
    Upnp_Icon **icons;

    /** Pointer to a the list of Upnp_Services belonging to this device.
     *  This is a null terminated list.  Every device must have at least one
     *  service, therefore this list may not be empty (or null).  */
    Upnp_Service **services;

    /** Pointer to any Upnp_Devices which are subdevices of to this device.
     *  This list a null terminated list.  If there are no sub-devices, this
     *  pointer should be set to null. */
    Upnp_Device **subdevices;

    /** Pointer to the root device structure. */
    Upnp_Device_Root *device_root;
};

/** The root device structure.  This structure is used by the underlying code
 *  to determine the URL base for the devices.
 *  @name Upnp_Device_Root
 *  @type typedef struct upnp_device_root
 */
struct upnp_device_root {
    /** An optional base url.  This is typcially left empty and filled in by
     * the library with the host address and port of the http server. */
    char base_URL[32];
    /** A pointer to the list of devices. */
    Upnp_Device *device;
};

extern Upnp_Device_Root **__upnp_root_devices;
#endif				/* _UPNP_H */
